// https://www.boost.org/doc/libs/1_92_0/libs/nowide/doc/html/index.html

#include <boost/nowide/args.hpp>
#include <boost/nowide/iostream.hpp>

#include <cassert>
#include <string>

int main(int argc, char **argv)
{
    // Converts argv to UTF-8 on Windows, a no-op elsewhere
    boost::nowide::args a(argc, argv);

    boost::nowide::cout << "argv[0] = " << argv[0] << std::endl;
    boost::nowide::cout << "Grüße aus Debian - UTF-8 output works" << std::endl;

    assert(argc >= 1);
    assert(std::string(argv[0]).size() > 0);

    boost::nowide::cerr.flush();
    boost::nowide::cout.flush();

    return 0;
}
