// boost1.67-1.67.0/libs/test/example/external_main_example_2.cpp

//  (C) Copyright Gennadiy Rozental 2001-2014.
//  Distributed under the Boost Software License, Version 1.0.
//  (See accompanying file LICENSE_1_0.txt or copy at 
//  http://www.boost.org/LICENSE_1_0.txt)

//  See http://www.boost.org/libs/test for the library home page.
//
// ***************************************************************************

#ifndef BOOST_TEST_DYN_LINK
#define BOOST_TEST_DYN_LINK
#endif
#include <boost/test/unit_test.hpp>
using namespace boost::unit_test;

//____________________________________________________________________________//

BOOST_AUTO_TEST_SUITE( test_suite_1 )

BOOST_AUTO_TEST_CASE( test_case_1 )
{
     BOOST_TEST_MESSAGE( "Testing is in progress" );

     BOOST_CHECK( true );
}

BOOST_AUTO_TEST_SUITE_END()

//____________________________________________________________________________//

bool
init_function()
{
    // do your own initialization here
    // if it successful return true

    // But, you CAN'T use testing tools here
    return true;
}

//____________________________________________________________________________//

int
main( int argc, char* argv[] )
{
    return ::boost::unit_test::unit_test_main( &init_function, argc, argv );
}

//____________________________________________________________________________//
